@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2003, 2005, 2007, 2008, 2010, 2011 Free
+   Copyright (C) 1999, 2001, 2003, 2005, 2007-2008, 2010-2012 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -75,7 +75,7 @@ imap4d_capability (struct imap4d_command *command, imap4d_tokbuf_t tok)
   
   io_sendf ("* CAPABILITY");
 
-  mu_list_do (capa_list, print_capa, NULL);
+  mu_list_foreach (capa_list, print_capa, NULL);
   
   imap4d_auth_capability ();
   io_sendf ("\n");
