@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,10 +31,13 @@
 
 struct fetch_runtime_closure
 {
-  int eltno;
-  size_t msgno;
-  mu_message_t msg;
-  char *err_text;
+  int eltno;           /* Serial number of the last output FETCH element */
+  size_t msgno;        /* Sequence number of the message being processed */
+  mu_message_t msg;    /* The message itself */
+  mu_list_t msglist;   /* A list of referenced messages.  See KLUDGE below. */
+  char *err_text;      /* On return: error description if failed. */
+
+  mu_list_t fnlist;
 };
 
 struct fetch_function_closure;
@@ -61,8 +63,7 @@ struct fetch_parse_closure
 {
   int isuid;
   mu_list_t fnlist;
-  size_t *set;
-  int count;
+  mu_msgset_t msgset;
 };
 
 
@@ -310,6 +311,8 @@ fetch_envelope0 (mu_message_t msg)
   return RESP_OK;
 }
 
+static int fetch_bodystructure0 (mu_message_t message, int extension);
+
 /* The basic fields of a non-multipart body part are in the following order:
    body type:
    A string giving the content media type name as defined in [MIME-IMB].
@@ -368,8 +371,9 @@ bodystructure (mu_message_t msg, int extension)
   if (mu_header_aget_value (header, MU_HEADER_CONTENT_TYPE, &buffer) == 0)
     {
       struct mu_wordsplit ws;
-      char *s, *p;
-	  
+      char *p;
+      size_t len;
+      
       ws.ws_delim = " \t\r\n;=";
       ws.ws_alloc_die = imap4d_ws_alloc_die;
       if (mu_wordsplit (buffer, &ws, IMAP4D_WS_FLAGS))
@@ -379,25 +383,23 @@ bodystructure (mu_message_t msg, int extension)
 	  return RESP_BAD; /* FIXME: a better error handling, maybe? */
 	}
 
+      len = strcspn (ws.ws_wordv[0], "/");
       if (mu_c_strcasecmp (ws.ws_wordv[0], "MESSAGE/RFC822") == 0)
         message_rfc822 = 1;
-      else if (mu_c_strcasecmp (ws.ws_wordv[0], "TEXT/PLAIN") == 0)
+      else if (mu_c_strncasecmp (ws.ws_wordv[0], "TEXT", len) == 0)
         text_plain = 1;
 
-      s = strchr (ws.ws_wordv[0], '/');
-      if (s)
-	*s++ = 0;
+      ws.ws_wordv[0][len++] = 0;
       p = ws.ws_wordv[0];
       io_send_qstring (p);
       io_sendf (" ");
-      io_send_qstring (s);
+      io_send_qstring (ws.ws_wordv[0] + len);
 
       /* body parameter parenthesized list: Content-type attributes */
-      if (ws.ws_wordc > 1 || text_plain)
+      if (ws.ws_wordc > 1)
 	{
 	  int space = 0;
 	  char *lvalue = NULL;
-	  int have_charset = 0;
 	  int i;
 	  
 	  io_sendf (" (");
@@ -429,8 +431,6 @@ bodystructure (mu_message_t msg, int extension)
 		  
 		default:
 		  lvalue = ws.ws_wordv[i];
-		  if (mu_c_strcasecmp (lvalue, "charset") == 0)
-		    have_charset = 1;
 		}
 	    }
 	  
@@ -440,13 +440,7 @@ bodystructure (mu_message_t msg, int extension)
 		io_sendf (" ");
 	      io_send_qstring (lvalue);
 	    }
-	  
-	  if (!have_charset && text_plain)
-	    {
-	      if (space)
-		io_sendf (" ");
-	      io_sendf ("\"CHARSET\" \"US-ASCII\"");
-	    }
+
 	  io_sendf (")");
 	}
       else
@@ -497,7 +491,7 @@ bodystructure (mu_message_t msg, int extension)
       io_sendf (")");
       /* Add body structure of the encapsulated message.  */
       io_sendf ("(");
-      bodystructure (emsg, extension);
+      fetch_bodystructure0 (emsg, extension);
       io_sendf (")");
       /* Size in text lines of the encapsulated message.  */
       mu_message_lines (emsg, &lines);
@@ -568,7 +562,6 @@ fetch_bodystructure0 (mu_message_t message, int extension)
 
       mu_message_get_header (message, &header);
 
-
       /* The subtype.  */
       if (mu_header_aget_value (header, MU_HEADER_CONTENT_TYPE, &buffer) == 0)
 	{
@@ -671,7 +664,7 @@ set_seen (struct fetch_function_closure *ffc,
 	}
     }
 }
-
+
 static mu_message_t 
 fetch_get_part (struct fetch_function_closure *ffc,
 		struct fetch_runtime_closure *frt)
@@ -684,6 +677,111 @@ fetch_get_part (struct fetch_function_closure *ffc,
       return NULL;
   return msg;
 }
+
+/* FIXME: This is a KLUDGE.
+
+   There is so far no way to unref the MU elements being used to construct
+   a certain entity when this entity itself is being destroyed.  In particular,
+   retrieving a nested message/rfc822 part involves creating several messages
+   while unencapsulating, which messages should be unreferenced when the
+   topmost one is destroyed.
+
+   A temporary solution used here is to keep a list of such messages and
+   unreference them together with the topmost one when no longer needed.
+   A message is added to the list by frt_register_message().  All messages
+   in the list are unreferenced by calling frt_unregister_messages().
+
+   The proper solution is of course providing a way for mu_message_t (and
+   other MU objects) to unreference its parent elements.  This should be fixed
+   in later releases.
+ */
+
+static void
+_unref_message_item (void *data)
+{
+  mu_message_unref ((mu_message_t)data);
+}
+
+static int
+frt_register_message (struct fetch_runtime_closure *frt, mu_message_t msg)
+{
+  if (!frt->msglist)
+    {
+      int rc = mu_list_create (&frt->msglist);
+      if (rc)
+	return rc;
+      mu_list_set_destroy_item (frt->msglist, _unref_message_item);
+    }
+  return mu_list_append (frt->msglist, msg);
+}
+
+static void
+frt_unregister_messages (struct fetch_runtime_closure *frt)
+{
+  mu_list_clear (frt->msglist);
+}
+
+static mu_message_t 
+fetch_get_part_rfc822 (struct fetch_function_closure *ffc,
+		       struct fetch_runtime_closure *frt)
+{
+  mu_message_t msg = frt->msg, retmsg = NULL;
+  size_t i;
+  mu_header_t header;
+  const char *hval;
+  
+  if (ffc->nset == 0)
+    {
+      mu_message_ref (msg);
+      return msg;
+    }
+  
+  for (i = 0; i < ffc->nset; i++)
+    {
+      if (mu_message_get_part (msg, ffc->section_part[i], &msg))
+	return NULL;
+
+      if (mu_message_get_header (msg, &header))
+	return NULL;
+  
+      if (mu_header_sget_value (header, MU_HEADER_CONTENT_TYPE, &hval) == 0)
+	{
+	  struct mu_wordsplit ws;
+	  int rc;
+      
+	  ws.ws_delim = " \t\r\n;=";
+	  ws.ws_alloc_die = imap4d_ws_alloc_die;
+	  if (mu_wordsplit (hval, &ws, IMAP4D_WS_FLAGS))
+	    {
+	      mu_error (_("%s failed: %s"), "mu_wordsplit",
+			mu_wordsplit_strerror (&ws));
+	      return NULL;
+	    }
+
+	  rc = mu_c_strcasecmp (ws.ws_wordv[0], "MESSAGE/RFC822");
+	  mu_wordsplit_free (&ws);
+
+	  if (rc == 0)
+	    {
+	      rc = mu_message_unencapsulate  (msg, &retmsg, NULL);
+	      if (rc)
+		{
+		  mu_error (_("%s failed: %s"), "mu_message_unencapsulate",
+			    mu_strerror (rc));
+		  return NULL;
+		}
+	      if (frt_register_message (frt, retmsg))
+		{
+		  frt_unregister_messages (frt);
+		  return NULL;
+		}
+	      msg = retmsg;
+	    }
+	}
+    }
+  
+  return retmsg;
+}
 
 static void
 fetch_send_section_part (struct fetch_function_closure *ffc,
@@ -742,20 +840,21 @@ fetch_io (mu_stream_t stream, size_t start, size_t size, size_t max)
       io_sendf ("<%lu>", (unsigned long) start);
       io_sendf (" \"\"");
     }
-  else if (size + 2 < size) /* Check for integer overflow */
-    {
-      mu_stream_destroy (&rfc);
-      return RESP_BAD;
-    }
   else
     {
       int rc;
       char *buffer, *p;
       size_t total = 0;
 
-      p = buffer = malloc (size + 1);
-      if (!p)
-	imap4d_bye (ERR_NO_MEM);
+      if (size > max)
+	size = max;
+      if (size + 2 < size) /* Check for integer overflow */
+	{
+	  mu_stream_destroy (&rfc);
+	  return RESP_BAD;
+	}
+      
+      p = buffer = mu_alloc (size + 1);
 
       rc = mu_stream_seek (rfc, start, MU_SEEK_SET, NULL);
       if (rc)
@@ -860,17 +959,13 @@ _frt_internaldate (struct fetch_function_closure *ffc,
   const char *date;
   mu_envelope_t env = NULL;
   struct tm tm, *tmp = NULL;
-  mu_timezone tz;
-  char datebuf[sizeof ("13-Jul-2002 00:00:00")];
-  int tzoff;
-  int tzh = 0, tzm = 0;
+  struct mu_timezone tz;
   
   mu_message_get_envelope (frt->msg, &env);
   if (mu_envelope_sget_date (env, &date) == 0
-      && mu_parse_ctime_date_time (&date, &tm, &tz) == 0)
+      && mu_scan_datetime (date, MU_DATETIME_FROM, &tm, &tz, NULL) == 0)
     {
       tmp = &tm;
-      tzoff = tz.utc_offset;
     }
   else
     {
@@ -878,18 +973,13 @@ _frt_internaldate (struct fetch_function_closure *ffc,
       struct timeval stv;
       struct timezone stz;
       
-      gettimeofday(&stv, &stz);
+      gettimeofday (&stv, &stz);
       t = stv.tv_sec;
-      tzoff = - stz.tz_minuteswest;
+      tz.utc_offset = - stz.tz_minuteswest;
       tmp = localtime (&t);
     }
-  tzh = tzoff / 3600;
-  if (tzoff < 0)
-    tzoff = - tzoff;
-  tzm = tzoff % 3600;
-  mu_strftime (datebuf, sizeof (datebuf), "%d-%b-%Y %H:%M:%S", tmp);
   io_sendf ("%s", ffc->name);
-  io_sendf (" \"%s %+03d%02d\"", datebuf, tzh, tzm);
+  mu_c_streamftime (iostream, " \"%d-%b-%Y %H:%M:%S %z\"", tmp, &tz);
   return 0;
 }
 
@@ -918,7 +1008,7 @@ static int
 _frt_body (struct fetch_function_closure *ffc,
 	   struct fetch_runtime_closure *frt)
 {
-  mu_message_t msg;
+  mu_message_t msg = frt->msg;
   mu_stream_t stream = NULL;
   size_t size = 0, lines = 0;
   int rc;
@@ -928,12 +1018,6 @@ _frt_body (struct fetch_function_closure *ffc,
     io_sendf ("%s", ffc->name);
   else
     fetch_send_section_part (ffc, NULL, 1);
-  msg = fetch_get_part (ffc, frt);
-  if (!msg)
-    {
-      io_sendf (" \"\"");
-      return RESP_OK;
-    }
   mu_message_get_streamref (msg, &stream);
   mu_message_size (msg, &size);
   mu_message_lines (msg, &lines);
@@ -942,6 +1026,39 @@ _frt_body (struct fetch_function_closure *ffc,
   return rc;
 }
 
+/* BODY[N] */
+static int
+_frt_body_n (struct fetch_function_closure *ffc,
+	     struct fetch_runtime_closure *frt)
+{
+  mu_message_t msg;
+  mu_body_t body = NULL;
+  mu_stream_t stream = NULL;
+  size_t size = 0, lines = 0;
+  int rc;
+  
+  set_seen (ffc, frt);
+  if (ffc->name)
+    io_sendf ("%s",  ffc->name);
+  else
+    fetch_send_section_part (ffc, ffc->section_tag, 1);
+  msg = fetch_get_part (ffc, frt);
+  if (!msg)
+    {
+      io_sendf (" NIL");
+      return RESP_OK;
+    }
+
+  mu_message_get_body (msg, &body);
+  mu_body_size (body, &size);
+  mu_body_lines (body, &lines);
+  mu_body_get_streamref (body, &stream);
+  rc = fetch_io (stream, ffc->start, ffc->size, size + lines);
+  mu_stream_destroy (&stream);
+
+  return rc;
+}
+
 static int
 _frt_body_text (struct fetch_function_closure *ffc,
 		struct fetch_runtime_closure *frt)
@@ -957,10 +1074,10 @@ _frt_body_text (struct fetch_function_closure *ffc,
     io_sendf ("%s",  ffc->name);
   else
     fetch_send_section_part (ffc, ffc->section_tag, 1);
-  msg = fetch_get_part (ffc, frt);
+  msg = fetch_get_part_rfc822 (ffc, frt);
   if (!msg)
     {
-      io_sendf (" \"\"");
+      io_sendf (" NIL");
       return RESP_OK;
     }
 
@@ -970,6 +1087,7 @@ _frt_body_text (struct fetch_function_closure *ffc,
   mu_body_get_streamref (body, &stream);
   rc = fetch_io (stream, ffc->start, ffc->size, size + lines);
   mu_stream_destroy (&stream);
+  frt_unregister_messages (frt);
   return rc;
 }
 
@@ -1002,10 +1120,10 @@ _frt_header (struct fetch_function_closure *ffc,
   else
     fetch_send_section_part (ffc, ffc->section_tag, 1);
 
-  msg = fetch_get_part (ffc, frt);
+  msg = fetch_get_part_rfc822 (ffc, frt);
   if (!msg)
     {
-      io_sendf (" \"\"");
+      io_sendf (" NIL");
       return RESP_OK;
     }
   mu_message_get_header (msg, &header);
@@ -1014,6 +1132,39 @@ _frt_header (struct fetch_function_closure *ffc,
   mu_header_get_streamref (header, &stream);
   rc = fetch_io (stream, ffc->start, ffc->size, size + lines);
   mu_stream_destroy (&stream);
+  frt_unregister_messages (frt);
+  return rc;
+}
+
+static int
+_frt_mime (struct fetch_function_closure *ffc,
+	     struct fetch_runtime_closure *frt)
+{
+  mu_message_t msg;
+  mu_header_t header = NULL;
+  mu_stream_t stream = NULL;
+  size_t size = 0, lines = 0;
+  int rc;
+  
+  set_seen (ffc, frt);
+  if (ffc->name)
+    io_sendf ("%s",  ffc->name);
+  else
+    fetch_send_section_part (ffc, ffc->section_tag, 1);
+
+  msg = fetch_get_part (ffc, frt);
+  if (!msg)
+    {
+      io_sendf (" NIL");
+      return RESP_OK;
+    }
+  mu_message_get_header (msg, &header);
+  mu_header_size (header, &size);
+  mu_header_lines (header, &lines);
+  mu_header_get_streamref (header, &stream);
+  rc = fetch_io (stream, ffc->start, ffc->size, size + lines);
+  mu_stream_destroy (&stream);
+
   return rc;
 }
 
@@ -1057,13 +1208,13 @@ _frt_header_fields (struct fetch_function_closure *ffc,
     io_sendf (".NOT");
   io_sendf (" (");
   status = 0;
-  mu_list_do (ffc->headers, _send_header_name, &status);
+  mu_list_foreach (ffc->headers, _send_header_name, &status);
   io_sendf (")]");
   
-  msg = fetch_get_part (ffc, frt);
+  msg = fetch_get_part_rfc822 (ffc, frt);
   if (!msg)
     {
-      io_sendf (" \"\"");
+      io_sendf (" NIL");
       return RESP_OK;
     }
 
@@ -1071,7 +1222,8 @@ _frt_header_fields (struct fetch_function_closure *ffc,
   if (mu_message_get_header (msg, &header)
       || mu_header_get_iterator (header, &itr))
     {
-      io_sendf (" \"\"");
+      frt_unregister_messages (frt);
+      io_sendf (" NIL");
       return RESP_OK;
     }
 
@@ -1109,6 +1261,7 @@ _frt_header_fields (struct fetch_function_closure *ffc,
   mu_stream_seek (stream, 0, MU_SEEK_SET, NULL);
   status = fetch_io (stream, ffc->start, ffc->size, size + lines);
   mu_stream_destroy (&stream);
+  frt_unregister_messages (frt);
   
   return status;
 }
@@ -1143,9 +1296,7 @@ _do_fetch (void *item, void *data)
 static void
 append_ffc (struct fetch_parse_closure *p, struct fetch_function_closure *ffc)
 {
-  struct fetch_function_closure *new_ffc = malloc (sizeof (*new_ffc));
-  if (!new_ffc)
-    imap4d_bye (ERR_NO_MEM);
+  struct fetch_function_closure *new_ffc = mu_alloc (sizeof (*new_ffc));
   *new_ffc = *ffc;
   mu_list_append (p->fnlist, new_ffc);
 }
@@ -1347,7 +1498,7 @@ parse_section_text (imap4d_parsebuf_t p, struct fetch_function_closure *ffc,
   else if (allow_mime && mu_c_strcasecmp (p->token, "MIME") == 0)
     {
       imap4d_parsebuf_next (p, 1);
-      ffc->fun = _frt_header;
+      ffc->fun = _frt_mime;
       ffc->section_tag = "MIME";
     }
   else
@@ -1436,6 +1587,8 @@ parse_section (imap4d_parsebuf_t p, struct fetch_function_closure *ffc)
 	      imap4d_parsebuf_next (p, 1);
 	      parse_section_text (p, ffc, 1);
 	    }
+	  else
+	    ffc->fun = _frt_body_n;
 	}
       else
 	imap4d_parsebuf_exit (p, "Syntax error");
@@ -1595,39 +1748,23 @@ static int
 fetch_thunk (imap4d_parsebuf_t pb)
 {
   int status;
-  char *msgset;
+  char *mstr;
+  char *end;
   struct fetch_parse_closure *pclos = imap4d_parsebuf_data (pb);
   
-  msgset = imap4d_parsebuf_next (pb, 1);
-
-  /* Get the message numbers in set[].  */
-  status = util_msgset (msgset, &pclos->set, &pclos->count, pclos->isuid);
-  switch (status)
-    {
-    case 0:
-      /* Very good! */
-      break;
-      
-    case EINVAL:
-      /* RFC3501, section 6.4.8.
-	 
-	 A non-existent unique identifier is ignored without any error
-	 message generated.  Thus, it is possible for a UID FETCH command
-	 to return an OK without any data or a UID COPY or UID STORE to
-	 return an OK without performing any operations.
-
-	 Obviously the same holds true for non-existing message numbers
-	 as well, although I did not find any explicit mention thereof
-	 in the RFC. */
-      
-      pb->err_text = "Completed";
-      return RESP_OK;
-
-    default:
-      pb->err_text = "Failed to parse message set";
-      return RESP_NO;
-    }
+  mstr = imap4d_parsebuf_next (pb, 1);
 
+  status = mu_msgset_create (&pclos->msgset, mbox, MU_MSGSET_NUM);
+  if (status)
+    imap4d_parsebuf_exit (pb, "Software error");
+  
+  /* Parse sequence numbers. */
+  status = mu_msgset_parse_imap (pclos->msgset,
+				 pclos->isuid ? MU_MSGSET_UID : MU_MSGSET_NUM,
+				 mstr, &end);
+  if (status)
+    imap4d_parsebuf_exit (pb, "Failed to parse message set");
+  
   /* Compile the expression */
 
   /* Server implementations MUST implicitly
@@ -1641,6 +1778,23 @@ fetch_thunk (imap4d_parsebuf_t pb)
   return RESP_OK;
 }
 
+int
+_fetch_from_message (size_t msgno, mu_message_t msg, void *data)
+{
+  int rc = 0;
+  struct fetch_runtime_closure *frc = data;
+
+  frc->msgno = msgno;
+  frc->msg = msg;
+
+  io_sendf ("* %lu FETCH (", (unsigned long) msgno);
+  frc->eltno = 0;
+  rc = mu_list_foreach (frc->fnlist, _do_fetch, frc);
+  io_sendf (")\n");
+
+  return rc;
+}
+
 /* Where the real implementation is.  It is here since UID command also
    calls FETCH.  */
 int
@@ -1667,30 +1821,20 @@ imap4d_fetch0 (imap4d_tokbuf_t tok, int isuid, char **err_text)
 
   if (rc == RESP_OK)
     {
-      size_t i;
       struct fetch_runtime_closure frc;
-      
+
+      memset (&frc, 0, sizeof (frc));
+      frc.fnlist = pclos.fnlist;
       /* Prepare status code. It will be replaced if an error occurs in the
 	 loop below */
       frc.err_text = "Completed";
-  
-      for (i = 0; i < pclos.count && rc == RESP_OK; i++)
-	{
-	  frc.msgno = (isuid) ? uid_to_msgno (pclos.set[i]) : pclos.set[i];
-	  
-	  if (frc.msgno &&
-	      mu_mailbox_get_message (mbox, frc.msgno, &frc.msg) == 0)
-	    {
-	      io_sendf ("* %lu FETCH (", (unsigned long) frc.msgno);
-	      frc.eltno = 0;
-	      rc = mu_list_do (pclos.fnlist, _do_fetch, &frc);
-	      io_sendf (")\n");
-	    }
-	}
-      }
+
+      mu_msgset_foreach_message (pclos.msgset, _fetch_from_message, &frc);
+      mu_list_destroy (&frc.msglist);
+    }
   
   mu_list_destroy (&pclos.fnlist);
-  free (pclos.set);
+  mu_msgset_free (pclos.msgset);
   return rc;
 }
 
