@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-   2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,18 +51,22 @@ int ident_port;
 char *ident_keyfile;
 int ident_encrypt_only;
 
+int test_mode;
+
 const char *program_version = "imap4d (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU imap4d -- the IMAP4D daemon.");
 
 #define OPT_PREAUTH         259
 #define OPT_FOREGROUND      260
+#define OPT_TEST_MODE       261
 
 static struct argp_option options[] = {
   { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), 0},
   { "inetd",  'i', 0, 0, N_("run in inetd mode"), 0},
   { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
     N_("runs in daemon mode with a maximum of NUMBER children"), 0 },
-
+  { "test", OPT_TEST_MODE, 0, 0,
+    N_("run in test mode"), 0 },
   { "preauth", OPT_PREAUTH, NULL, 0,
     N_("start in preauth mode") },
   
@@ -119,6 +122,11 @@ imap4d_parse_opt (int key, char *arg, struct argp_state *state)
     case OPT_PREAUTH:
       preauth_mode = preauth_stdio;
       break;
+
+    case OPT_TEST_MODE:
+      mu_argp_node_list_new (lst, "mode", "inetd");
+      test_mode = 1;
+      break;
       
     case ARGP_KEY_INIT:
       mu_argp_node_list_init (&lst);
@@ -200,7 +208,7 @@ cb_preauth (void *data, mu_config_value_t *val)
     return parse_preauth_scheme (val->v.string, NULL);
   else if (val->v.string[0] == '/')
     {
-      preauth_program = xstrdup (val->v.string);
+      preauth_program = mu_strdup (val->v.string);
       preauth_mode = preauth_prog;
     }
   else
@@ -316,9 +324,10 @@ imap4d_session_setup0 ()
 
   if (modify_homedir)
     {
-      char *expr = mu_tilde_expansion (modify_homedir, "/", real_homedir);
+      char *expr = mu_tilde_expansion (modify_homedir, MU_HIERARCHY_DELIMITER,
+				       real_homedir);
       struct mu_wordsplit ws;
-      const char *env[3];
+      const char *env[5];
 
       env[0] = "user";
       env[1] = auth_data->name;
@@ -340,7 +349,7 @@ imap4d_session_setup0 ()
 	  mu_error (_("expanding %s yields empty string"), expr);
 	  return 1;
 	}
-      imap4d_homedir = strdup (ws.ws_wordv[0]);
+      imap4d_homedir = mu_strdup (ws.ws_wordv[0]);
       if (!imap4d_homedir)
 	{
 	  mu_error ("%s", mu_strerror (errno));
@@ -348,7 +357,7 @@ imap4d_session_setup0 ()
 	}
     }
   else
-    imap4d_homedir = strdup (real_homedir);
+    imap4d_homedir = mu_strdup (real_homedir);
 
   if (strcmp (imap4d_homedir, real_homedir)
       && imap4d_check_home_dir (imap4d_homedir,
@@ -399,7 +408,70 @@ get_client_address (int fd, struct sockaddr_in *pcs)
     }
   return 0;
 }
+
+static int
+set_strerr_flt ()
+{
+  mu_stream_t flt, trans[2];
+  int rc;
+  
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
+  if (rc == 0)
+    {
+      char sessidstr[10];
+      char *argv[] = { "inline-comment", NULL, "-S", NULL };
 
+      snprintf (sessidstr, sizeof sessidstr, "%08lx:", mu_session_id);
+      argv[1] = sessidstr;
+      rc = mu_filter_create_args (&flt, trans[0], "inline-comment", 3,
+				  (const char **)argv,
+				  MU_FILTER_ENCODE, MU_STREAM_WRITE);
+      mu_stream_unref (trans[0]);
+      if (rc == 0)
+	{
+	  mu_stream_set_buffer (flt, mu_buffer_line, 0);
+	  trans[0] = flt;
+	  trans[1] = NULL;
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM,
+				MU_IOCTL_OP_SET, trans);
+	  mu_stream_unref (trans[0]);
+	  if (rc)
+	    mu_error (_("%s failed: %s"), "MU_IOCTL_SET_STREAM",
+		      mu_stream_strerror (mu_strerr, rc));
+	}
+      else
+	mu_error (_("cannot create log filter stream: %s"), mu_strerror (rc));
+    }
+  else
+    {
+      mu_error (_("%s failed: %s"), "MU_IOCTL_GET_STREAM",
+		mu_stream_strerror (mu_strerr, rc));
+    }
+  return rc;
+}
+
+static void
+clr_strerr_flt ()
+{
+  mu_stream_t flt, trans[2];
+  int rc;
+
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
+  if (rc == 0)
+    {
+      flt = trans[0];
+
+      rc = mu_stream_ioctl (flt, MU_IOCTL_TOPSTREAM, MU_IOCTL_OP_GET, trans);
+      if (rc == 0)
+	{
+	  mu_stream_unref (trans[0]);
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_TOPSTREAM,
+				MU_IOCTL_OP_SET, trans);
+	  if (rc == 0)
+	    mu_stream_unref (flt);
+	}
+    }
+}
 
 void
 imap4d_child_signal_setup (RETSIGTYPE (*handler) (int signo))
@@ -414,9 +486,11 @@ imap4d_mainloop (int ifd, int ofd, int tls)
 {
   imap4d_tokbuf_t tokp;
   char *text;
-  int debug_mode = isatty (ifd);
   int signo;
 
+  if (!test_mode)
+    test_mode = isatty (ifd);
+
   if ((signo = setjmp (child_jmp)))
     {
       mu_diag_output (MU_DIAG_CRIT, _("got signal `%s'"), strsignal (signo));
@@ -453,10 +527,10 @@ imap4d_mainloop (int ifd, int ofd, int tls)
 
   if (imap4d_preauth_setup (ifd) == 0)
     {
-      if (debug_mode)
+      if (test_mode)
 	{
-	  mu_diag_output (MU_DIAG_INFO, _("started in debugging mode"));
-	  text = "IMAP4rev1 Debugging mode";
+	  mu_diag_output (MU_DIAG_INFO, _("started in test mode"));
+	  text = "IMAP4rev1 Test mode";
 	}
       else
 	text = "IMAP4rev1";
@@ -496,10 +570,21 @@ imap4d_connection (int fd, struct sockaddr *sa, int salen,
 		   struct mu_srv_config *pconf, void *data)
 {
   struct imap4d_srv_config *cfg = (struct imap4d_srv_config *) pconf;
-
+  int rc;
+  
   idle_timeout = pconf->timeout;
   imap4d_transcript = pconf->transcript;
+
+  if (mu_log_session_id)
+    rc = set_strerr_flt ();
+  else
+    rc = 1;
+  
   imap4d_mainloop (fd, fd, cfg->tls);
+
+  if (rc == 0)
+    clr_strerr_flt ();
+  
   return 0;
 }
 
@@ -542,8 +627,13 @@ imap4d_master_signal (int signo)
   longjmp (master_jmp, signo);
 }
 
-
+static void
+imap4d_alloc_die ()
+{
+  imap4d_bye (ERR_NO_MEM);
+}
 
+
 int
 main (int argc, char **argv)
 {
@@ -589,6 +679,8 @@ main (int argc, char **argv)
   mu_m_server_set_timeout (server, 1800);  /* RFC2060: 30 minutes. */
   mu_m_server_set_strexit (server, mu_strexit);
   
+  mu_alloc_die_hook = imap4d_alloc_die;
+
   mu_log_syslog = 1;
 
   if (mu_app_init (&argp, imap4d_capa, imap4d_cfg_param, 
@@ -603,9 +695,12 @@ main (int argc, char **argv)
 #endif
 
   namespace_init ();
-  
-  auth_gssapi_init ();
-  auth_gsasl_init ();
+
+  if (mu_gsasl_enabled ())
+    {
+      auth_gssapi_init ();
+      auth_gsasl_init ();
+    }
 
 #ifdef USE_LIBPAM
   if (!mu_pam_service)
