@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007, 2008, 2009, 2010, 2011 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -37,7 +36,6 @@ imap4d_lsub (struct imap4d_command *command, imap4d_tokbuf_t tok)
   char *ref;
   char *wcard;
   char *pattern;
-  const char *delim = "/";
   mu_property_t prop;
   mu_iterator_t itr;
   
@@ -64,9 +62,9 @@ imap4d_lsub (struct imap4d_command *command, imap4d_tokbuf_t tok)
 	  
 	  mu_iterator_current_kv (itr, (const void **)&name, (void**)&val);
 
-	  if (util_wcard_match (name, pattern, delim) == 0)
-	    io_untagged_response (RESP_NONE, "LSUB () \"%s\" \"%s\"",
-				  delim, name);
+	  if (mu_imap_wildmatch (pattern, name, MU_HIERARCHY_DELIMITER) == 0)
+	    io_untagged_response (RESP_NONE, "LSUB () \"%c\" \"%s\"",
+				  MU_HIERARCHY_DELIMITER, name);
 	}
     }
   else
