@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007, 2008, 2009, 2010, 2011 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007-2012 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ print_namespace (int nsid)
       int count;
       count = 0;
       io_sendf ("(");
-      mu_list_do (list, print_namespace_fun, &count);
+      mu_list_foreach (list, print_namespace_fun, &count);
       io_sendf (")");
     }
 }
@@ -84,7 +84,7 @@ namespace_enumerate (int id, nsfp_t f, void *closure)
   nsc.fun = f;
   nsc.closure = closure;
   return namespace[id] == 0 ? 0 :
-          mu_list_do (namespace[id], _enum_fun, &nsc);
+          mu_list_foreach (namespace[id], _enum_fun, &nsc);
 }
 
 static int
@@ -166,7 +166,7 @@ risky_pattern (const char *pattern, int delim)
 
 char *
 namespace_checkfullpath (const char *name, const char *pattern, 
-			 const char *delim, int *nspace)
+			 int *nspace)
 {
   struct namespace_info info;
   const char *p;
@@ -185,7 +185,7 @@ namespace_checkfullpath (const char *name, const char *pattern,
       name = p + 1;
     }
 
-  path = util_getfullpath (name, delim);
+  path = util_getfullpath (name);
   if (!path)
     {
       free (scheme);
@@ -226,17 +226,17 @@ namespace_checkfullpath (const char *name, const char *pattern,
 }
 
 char *
-namespace_getfullpath (const char *name, const char *delim, int *nspace)
+namespace_getfullpath (const char *name, int *nspace)
 {
   char *ret;
   if (mu_c_strcasecmp (name, "INBOX") == 0 && auth_data->change_uid)
     {
-      ret = strdup (auth_data->mailbox);
+      ret = mu_strdup (auth_data->mailbox);
       if (nspace)
 	*nspace = NS_PRIVATE;
     }
   else
-    ret = namespace_checkfullpath (name, NULL, delim, nspace);
+    ret = namespace_checkfullpath (name, NULL, nspace);
   return ret;
 }
 
@@ -271,7 +271,7 @@ namespace_init ()
 	  mu_list_t list;
 	  mu_list_create (&list);
 	  mu_list_set_destroy_item (list, mu_list_free_item);
-	  mu_list_do (namespace[i], normalize_fun, list);
+	  mu_list_foreach (namespace[i], normalize_fun, list);
 	  mu_list_set_destroy_item (namespace[i], mu_list_free_item);
 	  mu_list_destroy (&namespace[i]);
 	  namespace[i] = list;
