@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2003, 2005, 2006, 2007, 2008, 2009, 2010,
-   2011 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2003, 2005-2012 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ imap4d_select0 (struct imap4d_command *command, const char *mboxname,
 
   if (strcmp (mboxname, "INBOX") == 0)
     flags |= MU_STREAM_CREAT;
-  mailbox_name = namespace_getfullpath (mboxname, "/", NULL);
+  mailbox_name = namespace_getfullpath (mboxname, NULL);
 
   if (!mailbox_name)
     return io_completion_response (command, RESP_NO, "Couldn't open mailbox");
@@ -137,14 +137,14 @@ imap4d_select_status ()
   io_untagged_response (RESP_OK, "[UIDNEXT %lu] Predicted next uid",
 	                   (unsigned long) uidnext);
   if (unseen)
-    io_untagged_response (RESP_OK, "[UNSEEN %lu] first unseen messsage ",
+    io_untagged_response (RESP_OK, "[UNSEEN %lu] first unseen messsage",
 	                     (unsigned long) unseen);
   io_untagged_response (RESP_NONE, "FLAGS (%s)", mflags);
   /* FIXME:
      - '\*' can be supported if we use the attribute_set userflag()
      - Answered is still not set in the mailbox code.  */
   if (!(select_flags & MU_STREAM_WRITE))
-    io_untagged_response (RESP_OK, "[PERMANENTFLAGS ()] No Permanent flags");
+    io_untagged_response (RESP_OK, "[PERMANENTFLAGS ()] No permanent flags");
   else
     io_untagged_response (RESP_OK, "[PERMANENTFLAGS (%s)] Permanent flags",
                           mflags);
