@@ -1,6 +1,6 @@
 /* GNU mailutils - a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007, 2009, 2010, 2011 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001, 2007, 2009-2012 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
