@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2009, 2011-2012 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -25,7 +25,8 @@ struct _mu_message_stream
   mu_stream_t transport;  /* Actual stream */
   mu_off_t offset;
 
-  char *envelope;
+  int construct_envelope;
+  char *envelope_string;
   size_t envelope_length; 
   char *from;
   char *date;
