@@ -1,6 +1,5 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009, 2010 Free Software Foundation,
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006-2007, 2009-2012 Free Software Foundation,
    Inc.
 
    This library is free software; you can redistribute it and/or
@@ -14,10 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include <mailutils/cpp/address.h>
 
