@@ -103,8 +103,8 @@
 #line 1 "mh_fmtgram.y"
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2004, 2007, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2004, 2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -2164,8 +2164,8 @@ prog_reserve (size_t count)
     {
       size_t inc = (count + 1) / FORMAT_INC + 1;
       format.progsize += inc * FORMAT_INC;
-      format.prog = xrealloc (format.prog,
-			      format.progsize * sizeof format.prog[0]);
+      format.prog = mu_realloc (format.prog,
+			        format.progsize * sizeof format.prog[0]);
     }
 }
 
